@@ -1,29 +1,38 @@
 import collections
 import datetime as dt
 from collections import namedtuple
-from dataclasses import dataclass
 from pathlib import Path
 from textwrap import dedent
 from typing import Dict, List, Tuple
 
 import cftime
 import six
+import xarray as xr
+from dataclasses import dataclass, replace
 from xclim.core.units import units2pint
 
 from .commands import (
     BasinIndexCommand,
     BasinStateVariablesCommand,
     ChannelProfileCommand,
+    DataCommand,
+    GaugeCommand,
+    GriddedForcingCommand,
+    GridWeightsCommand,
     HRUsCommand,
     HRUStateVariableTableCommand,
     LandUseClassesCommand,
+    MonthlyAverageCommand,
+    ObservationDataCommand,
     RainCorrection,
     RavenConfig,
     ReservoirCommand,
+    Routing,
     SBGroupPropertyMultiplierCommand,
     SnowCorrection,
     SoilClassesCommand,
     SoilProfilesCommand,
+    StationForcingCommand,
     SubBasinGroupCommand,
     SubBasinsCommand,
     VegetationClassesCommand,
@@ -32,6 +41,7 @@ from .commands import (
 HRU = HRUsCommand.Record
 HRUState = HRUStateVariableTableCommand.Record
 LU = LandUseClassesCommand.Record
+Sub = SubBasinsCommand.Record
 
 """
 Raven configuration
@@ -60,6 +70,7 @@ Simulation end date and duration are updated automatically when duration, start
 
 """
 
+# CF standard names used for mappings
 default_input_variables = (
     "pr",
     "rainfall",
@@ -71,6 +82,36 @@ default_input_variables = (
     "water_volume_transport_in_river_channel",
 )
 
+# Map CF-Convention standard name to Raven Forcing name
+forcing_names = {
+    "tasmin": "TEMP_MIN",
+    "tasmax": "TEMP_MAX",
+    "tas": "TEMP_AVE",
+    "rainfall": "RAINFALL",
+    "pr": "PRECIP",
+    "prsn": "SNOWFALL",
+    "evspsbl": "PET",
+    "water_volume_transport_in_river_channel": "HYDROGRAPH",
+}
+
+
+# Alternate typical variable names found in netCDF files, keyed by CF standard name
+alternate_nc_names = {
+    "tasmin": ["tasmin", "tmin"],
+    "tasmax": ["tasmax", "tmax"],
+    "tas": ["tas", "t2m"],
+    "rainfall": ["rainfall", "rain"],
+    "pr": ["pr", "precip", "prec", "precipitation", "tp"],
+    "prsn": ["prsn", "snow", "snowfall", "solid_precip"],
+    "evspsbl": ["pet", "evap", "evapotranspiration"],
+    "water_volume_transport_in_river_channel": [
+        "qobs",
+        "discharge",
+        "streamflow",
+        "dis",
+    ],
+}
+
 rain_snow_fraction_options = (
     "RAINSNOW_DATA",
     "RAINSNOW_DINGMAN",
@@ -97,8 +138,6 @@ evaporation_options = (
     "PET_OUDIN",
 )
 
-state_variables = ()
-
 calendar_options = (
     "PROLEPTIC_GREGORIAN",
     "JULIAN",
@@ -237,224 +276,82 @@ class RV(collections.abc.Mapping, RavenConfig):
                 self[key] = val
 
 
-class RavenNcData(RV):
-    pat = """
-          :{kind} {raven_name} {site} {runits}
-              :ReadFromNetCDF
-                 :FileNameNC      {path}
-                 :VarNameNC       {var_name}
-                 :DimNamesNC      {dimensions}
-                 :StationIdx      {index}
-                 {time_shift}
-                 {linear_transform}
-                 {deaccumulate}
-              :EndReadFromNetCDF
-          :End{kind}
-          """
-
-    _var_names = {
-        "tasmin": "TEMP_MIN",
-        "tasmax": "TEMP_MAX",
-        "tas": "TEMP_AVE",
-        "rainfall": "RAINFALL",
-        "pr": "PRECIP",
-        "prsn": "SNOWFALL",
-        "evspsbl": "PET",
-        "water_volume_transport_in_river_channel": "HYDROGRAPH",
-    }
-
-    _var_runits = {
-        "tasmin": "degC",
-        "tasmax": "degC",
-        "tas": "degC",
-        "pr": "mm/d",
-        "rainfall": "mm/d",
-        "prsn": "mm/d",
-        "evspsbl": "mm/d",
-        "water_volume_transport_in_river_channel": "m**3/s",
-    }
-
-    def __init__(self, **kwargs):
-        """Instantiate from attributes scrapped from the netCDF file."""
-        self.var = None
-        self.path = None
-        self.var_name = None
-        self.units = None
-        self.scale_factor = None
-        self.add_offset = None
-
-        self._time_shift = None
-        self._dimensions = None
-        self._index = None
-        self._linear_transform = None
-        self._kind = None
-        self._runits = None
-        self._raven_name = None
-        self._site = None
-        self._deaccumulate = None
-
-        super().__init__(**kwargs)
-
-    @property
-    def raven_name(self):
-        return self._var_names[self.var]
-
-    @property
-    def runits(self):
-        return self._var_runits[self.var]
-
-    @property
-    def kind(self):
-        if self.var == "water_volume_transport_in_river_channel":
-            return "ObservationData"
-        else:
-            return "Data"
-
-    @property
-    def site(self):
-        if self.var == "water_volume_transport_in_river_channel":
-            return 1
-        else:
-            return ""
-
-    @property
-    def index(self):
-        if self._index is not None:
-            return str(self._index + 1)
-
-    @index.setter
-    def index(self, value):
-        self._index = value
-
-    @property
-    def dimensions(self):
-        """Return dimensions as Raven expects it:
-        - time
-        - station time
-        - lon lat time
-        """
-        dims = list(self._dimensions)
-
-        # Move the time dimension at the end
-        dims.remove("time")
-        dims.append("time")
-
-        return " ".join(dims)
-
-    @dimensions.setter
-    def dimensions(self, value):
-        if "time" not in value:
-            raise ValueError("Raven expects a time dimension.")
-
-        self._dimensions = value
-
-        # If there is no spatial dimension, set the index to 0.
-        if self.index is None and len(value) == 1:
-            self._index = 0
-
-    @property
-    def time_shift(self):
-        """The fraction describing the time shift, for example to convert UTC to local time."""
-        if self._time_shift is not None:
-            return f":TimeShift {self._time_shift}"
-
-    @time_shift.setter
-    def time_shift(self, value):
-        self._time_shift = value
-
-    @property
-    def linear_transform(self):
-        """A sequence of two values: multiplicative factor and additive offset."""
-        lt = self._linear_transform
-        if lt is not None:
-            slope, intercept = lt or (1, 0)
-            return ":LinearTransform {:.15f} {:.15f}".format(slope, intercept)
-
-    @linear_transform.setter
-    def linear_transform(self, value):
-        """Set the scale factor and offset."""
-        if len(value) == 1:
-            value = (value, 0)
-        elif len(value) == 2:
-            pass
-        else:
-            raise ValueError(
-                "Linear transform takes at most two values: "
-                "a scaling factor and an offset."
-            )
-
-        self._linear_transform = value
-
-    @property
-    def deaccumulate(self):
-        """Convert cumulative precipitation to precipitation rate (mm/d)."""
-        return ":Deaccumulate" if self._deaccumulate is True else ""
-
-    @deaccumulate.setter
-    def deaccumulate(self, value):
-        self._deaccumulate = value
-
-    def _check_units(self):
-        import warnings
-
-        if units2pint(self.units) != units2pint(self.runits):
-            if self._linear_transform is None:
-                warnings.warn(
-                    f"Units are not what Raven expects for {self.var}.\n"
-                    f"Actual: {self.units}\n"
-                    f"Expected: {self.runits}\n"
-                    f"Make sure to set linear_transform to perform conversion."
-                )
-
-    def __str__(self):
-        if self.var is None:
-            return ""
-        else:
-            kwds = {k: v if v is not None else "" for (k, v) in self.items()}
-            return self.pat.format(**kwds)
-
-
-class MonthlyAverage(RV):
-    pat = ":MonthlyAve{var}, {data}"
-
-    def __init__(self, var=None, data=None):
-        self.var = var
-        self.data = data
-
-    def __str__(self):
-        if self.var is None:
-            return ""
-        out = self.pat.format(var=self.var, data=", ".join([str(d) for d in self.data]))
-        return out
-
-
 class RVT(RV):
     def __init__(self, **kwargs):
-        self._nc_index = None
-        self.gridded_forcings = ()
+        self.pr = {}
+        self.rainfall = {}
+        self.prsn = {}
+        self.tasmin = {}
+        self.tasmax = {}
+        self.tas = {}
+        self.evspsbl = {}
+        self.water_volume_transport_in_river_channel = {}
+
+        self.nc_index = None
+        self.gauge_latitude = None
+        self.gauge_longitude = None
+        self.gauge_elevation = None
+
         self.raincorrection = 1
         self.snowcorrection = 1
 
+        self.monthly_ave_evaporation = ()
+        self.monthly_ave_temperature = ()
+
+        self.gridded_forcings = ()
+
+        # For a distributed model these weights will be shared among all the StationForcing commands
+        self.grid_weights = None
+
+        self.var_cmds = {}
+        # Dictionary of potential variable names, keyed by CF standard name.
+        # http://cfconventions.org/Data/cf-standard-names/60/build/cf-standard-name-table.html
+        # PET is the potential evapotranspiration, while evspsbl is the actual evap.
+        # TODO: Check we're not mixing precip and rainfall.
+
         super(RVT, self).__init__(**kwargs)
 
     @property
-    def nc_index(self):
-        return self._nc_index
+    def variables(self):
+        return (getattr(self, name) for name in default_input_variables)
 
-    @nc_index.setter
-    def nc_index(self, value):
-        for key, val in self.items():
-            if isinstance(val, RavenNcData):
-                setattr(val, "index", value)
+    @property
+    def gauge(self):
+        data = [o for o in self.var_cmds.values() if isinstance(o, DataCommand)]
+        if data:
+            return GaugeCommand(
+                latitude=self.gauge_latitude,
+                longitude=self.gauge_longitude,
+                elevation=self.gauge_elevation,
+                raincorrection=self.raincorrection,
+                snowcorrection=self.snowcorrection,
+                monthly_ave_evaporation=self.monthly_ave_evaporation,
+                monthly_ave_temperature=self.monthly_ave_temperature,
+                data=data,
+            )
+        else:
+            return ""
+
+    @property
+    def station_forcing_list(self):
+        data = [
+            o for o in self.var_cmds.values() if isinstance(o, StationForcingCommand)
+        ]
+        return "\n\n".join(map(str, data))
 
     @property
     def gridded_forcing_list(self):
-        return "\n\n".join(map(str, self.gridded_forcings))
-        # return GriddedForcingList(self.gridded_forcings)
+        data = [
+            o for o in self.var_cmds.values() if isinstance(o, GriddedForcingCommand)
+        ]
+        # This is really a hack for now, as model.rvt.gridded_forcings are set
+        # directly by the user in TestRouting.test_lievre_tutorial
+        data += self.gridded_forcings
+        return "\n\n".join(map(str, data))
 
-    @gridded_forcing_list.setter
-    def gridded_forcing_list(self, value):
-        self.gridded_forcings = value
+    @property
+    def observed_data_cmd(self):
+        return self.var_cmds.get("water_volume_transport_in_river_channel", "")
 
     @property
     def raincorrection_cmd(self):
@@ -464,23 +361,6 @@ class RVT(RV):
     def snowcorrection_cmd(self):
         return SnowCorrection(self.snowcorrection)
 
-    def update(self, items, force=False):
-        """Update values from dictionary items.
-
-        Parameters
-        ----------
-        items : dict
-          Dictionary of values.
-        force : bool
-          If True, un-initialized keys can be set.
-        """
-        if force:
-            raise ValueError("Cannot add a new variable at run-time.")
-        else:
-            for key, val in items.items():
-                if isinstance(val, dict):
-                    self[key].update(val, force=True)
-
 
 class RVI(RV):
     def __init__(self, **kwargs):
@@ -491,6 +371,7 @@ class RVI(RV):
         self.longitude = None
         self.run_index = 0
         self.raven_version = "3.0.1 rev#275"
+        self.routing = "ROUTE_NONE"
 
         self._run_name = "run"
         self._start_date = None
@@ -624,6 +505,10 @@ class RVI(RV):
             raise ValueError
         self._suppress_output = value
 
+    @property
+    def routing_cmd(self):
+        return Routing(value=self.routing)
+
     @property
     def rain_snow_fraction(self):
         """Rain snow partitioning."""
@@ -743,10 +628,11 @@ class RVC(RV):
 class RVH(RV):
     subbasins: Tuple[SubBasinsCommand.Record] = (SubBasinsCommand.Record(),)
     land_subbasins: Tuple[int] = ()
+    land_subbasin_property_multiplier: SBGroupPropertyMultiplierCommand = ""
     lake_subbasins: Tuple[int] = ()
+    lake_subbasin_property_multiplier: SBGroupPropertyMultiplierCommand = ""
     reservoirs: Tuple[ReservoirCommand] = ()
     hrus: Tuple[HRUsCommand.Record] = ()
-    sb_group_property_multipliers: Tuple[SBGroupPropertyMultiplierCommand] = ()
 
     template = """
     {subbasins_cmd}
@@ -780,10 +666,6 @@ class RVH(RV):
     def hrus_cmd(self):
         return HRUsCommand(self.hrus)
 
-    @property
-    def sb_group_property_multiplier_list(self):
-        return "\n\n".join(map(str, self.sb_group_property_multipliers))
-
     def to_rv(self):
         # params = self.items()
         return dedent(self.template).format(**dict(self.items()))
@@ -797,6 +679,7 @@ class RVP(RV):
     vegetation_classes: Tuple[VegetationClassesCommand.Record] = ()
     land_use_classes: Tuple[LandUseClassesCommand.Record] = ()
     channel_profiles: Tuple[ChannelProfileCommand] = ()
+    avg_annual_runoff: float = 0
 
     @property
     def soil_classes_cmd(self):
@@ -946,13 +829,13 @@ def _parser(lines, indent="", fmt=str):
                     out[key][i] = dict(
                         index=i, name=name, **_parser(lines, new_indent + "  ", float)
                     )
-                elif key in ["Qlat", "Qout"]:
-                    n, *values, last = value.split(",")
-                    out[key] = list(map(float, values))
-                    out[key + "Last"] = float(last)
-                elif key == "Qin":
+                # elif key in ["Qlat", "Qout"]:
+                #     n, *values, last = value.split(",")
+                #     out[key] = list(map(float, values))
+                #     out[key + "Last"] = float(last)
+                elif key in ["Qin", "Qout", "Qlat"]:
                     n, *values = value.split(",")
-                    out[key] = list(map(float, values))
+                    out[key] = (int(n),) + tuple(map(float, values))
                 else:
                     out[key] = (
                         list(map(fmt, value.split(","))) if "," in value else fmt(value)
